@@ -36,6 +36,7 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <signal.h>
+#include <sys/signal.h>
 
 #define LINE_LENGTH 4096 /* max line length we'll grok */
 
@@ -50,6 +51,7 @@ typedef struct {
 	int read_fd;
 	int write_fd;
 	pid_t pid;
+	GnomeVFSFileInfoOptions info_opts;
 } SshHandle;
 
 static GnomeVFSResult do_open           (GnomeVFSMethod *method,
@@ -430,7 +432,7 @@ do_open_directory (GnomeVFSMethod *method,
 	if (result != GNOME_VFS_OK) {
 		return result;
 	}
-
+	handle->info_opts = options;
 	handle->open_mode = GNOME_VFS_OPEN_NONE;
 	handle->type = SSH_LIST;
 	*method_handle = (GnomeVFSMethodHandle *)handle;
@@ -446,6 +448,62 @@ do_close_directory (GnomeVFSMethod *method,
 	return ssh_destroy ((SshHandle *)method_handle);
 }
 
+static void
+get_access_info (GnomeVFSURI *uri, GnomeVFSFileInfo *file_info)
+{
+     gint i;
+     gchar *name;
+     gchar *quoted_name;
+     struct param {
+             char c;
+             GnomeVFSFilePermissions perm;
+     };
+     struct param params[2] = {{'r', GNOME_VFS_PERM_ACCESS_READABLE},
+                               {'w', GNOME_VFS_PERM_ACCESS_WRITABLE}};
+
+     name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
+
+
+     if ( *name == '\0' ) {
+             quoted_name = g_shell_quote ("/");
+     } else {
+             quoted_name = g_shell_quote (name);
+     }
+     g_free (name);
+
+     for (i = 0; i<2; i++) {
+             gchar c;
+             gchar *cmd;
+             SshHandle *handle;
+             GnomeVFSFileSize bytes_read;
+             GnomeVFSResult result;
+
+             cmd = g_strdup_printf ("test -%c %s && echo $?", 
+                                    params[i].c, quoted_name);
+             result = ssh_connect (&handle, uri, cmd);
+             g_free (cmd);
+             
+             if (result != GNOME_VFS_OK) {
+                     g_free(quoted_name);
+                     return;
+             }               
+             
+             result = ssh_read (handle, &c, 1, &bytes_read);
+             if ((bytes_read > 0) && (c == '0')) {
+                     file_info->permissions |= params[i].perm;
+             } else {
+                     file_info->permissions &= ~params[i].perm;
+             }
+                     
+             ssh_destroy (handle);
+     }
+
+     file_info->permissions &= ~GNOME_VFS_PERM_ACCESS_EXECUTABLE;
+     file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_ACCESS;
+
+     g_free(quoted_name);
+}
+
 static GnomeVFSResult 
 do_read_directory (GnomeVFSMethod *method,
 		   GnomeVFSMethodHandle *method_handle,
@@ -512,14 +570,24 @@ do_read_directory (GnomeVFSMethod *method,
 
 		/* FIXME: support symlinks correctly */
 
-		file_info->mime_type = g_strdup 
-			(gnome_vfs_get_file_mime_type (filename, &st, FALSE));
+		if (((SshHandle*)method_handle)->info_opts
+		    & GNOME_VFS_FILE_INFO_GET_MIME_TYPE) {
+			file_info->mime_type = g_strdup 
+				(gnome_vfs_get_file_mime_type (filename, &st, 
+							       FALSE));
+			file_info->valid_fields 
+				|= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
+		}
 
-		file_info->valid_fields |= GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE;
 		file_info->valid_fields &= 
 			~GNOME_VFS_FILE_INFO_FIELDS_BLOCK_COUNT;
 		file_info->valid_fields &= 
 			~GNOME_VFS_FILE_INFO_FIELDS_IO_BLOCK_SIZE;
+		if (((SshHandle*)method_handle)->info_opts 
+		    & GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS) {
+			get_access_info (((SshHandle*)method_handle)->uri, 
+					 file_info);
+		}
 
 		/* Break out.
 		   We are in a loop so we get the first 'ls' line;
@@ -561,7 +629,7 @@ do_get_file_info (GnomeVFSMethod *method,
 	if (result != GNOME_VFS_OK) {
 		return result;
 	}
-
+	handle->info_opts = options;
 	handle->open_mode = GNOME_VFS_OPEN_NONE;
 	handle->type = SSH_LIST;
 
